@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2007 2009 2011 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -75,6 +75,7 @@ static gint sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
 static gboolean sndfile_save_main(Chunk *chunk, gchar *filename, 
 				  int format, int dither_mode, StatusBar *bar,
 				  gboolean *fatal);
+static gboolean sndfile_ogg_flag = FALSE;
 #endif
 
 static Chunk *raw_load(gchar *filename, int dither_mode, StatusBar *bar);
@@ -163,10 +164,17 @@ static void setup_types(void)
 	  }
 	  if ((info.format&SF_FORMAT_TYPEMASK) == SF_FORMAT_RAW ||
 	      (info.format&SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV) continue;
+	  if ((info.format&SF_FORMAT_TYPEMASK) == SF_FORMAT_OGG)
+	       sndfile_ogg_flag=TRUE;
 	  snprintf(buf,sizeof(buf),".%s",info.extension);
 	  register_file_type((gchar *)info.name, buf, FALSE, sndfile_check, 
 			     sndfile_load, sndfile_save, 
 			     info.format&SF_FORMAT_TYPEMASK);
+	  if (!strcmp(info.extension,"oga"))
+	       register_file_type((gchar *)info.name, ".ogg", FALSE,
+				  sndfile_check,
+				  sndfile_load, sndfile_save,
+				  info.format&SF_FORMAT_TYPEMASK);
      }
 #endif
      if (program_exists("oggenc") || program_exists("oggdec"))
@@ -510,6 +518,7 @@ static Chunk *wav_load(char *filename, int dither_mode, StatusBar *bar)
      fmt.samplebytes = fmt.samplesize * fmt.channels;
      fmt.sign = (bits>8);
      fmt.bigendian = rifx;
+     fmt.packing = 0;
 
      ds = gtk_type_new(datasource_get_type());
 
@@ -545,23 +554,28 @@ static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
 
      /* Check if the format is OK */     
 
-#ifndef HAVE_LIBSNDFILE
-     if (chunk->format.type == DATAFORMAT_FLOAT &&
-	 !ieee_le_compatible) {
-	  user_error(_("On this system, libsndfile is required to save "
-		     "floating-point wav files."));
-	  return TRUE;
-     }
-#endif
-
      if (fmt->type == DATAFORMAT_PCM) {
 	  /* Check that the sign and endian-ness is correct */
 	  if (fmt->samplesize == 1) q = FALSE;
 	  else q = TRUE;
-	  if (XOR(fmt->sign,q) || fmt->bigendian) {
+	  if (XOR(fmt->sign,q) || fmt->bigendian || fmt->packing!=0) {
 	       memcpy(&cfmt,fmt,sizeof(cfmt));
 	       cfmt.sign = q;
 	       cfmt.bigendian = FALSE;
+	       if (cfmt.packing != 0) {
+		    g_assert(cfmt.samplesize == 4);
+		    cfmt.samplesize = 3;
+		    cfmt.packing = 0;
+	       }
+	       c = chunk_convert(chunk,&cfmt,DITHER_UNSPEC,bar);
+	       b = wav_save(c,filename,settings,type,dither_mode,bar,fatal);
+	       gtk_object_sink(GTK_OBJECT(c));
+	       return b;
+	  }
+     } else if (ieee_le_compatible || ieee_be_compatible) {
+	  if (fmt->bigendian) {
+	       memcpy(&cfmt,fmt,sizeof(cfmt));
+	       cfmt.bigendian = FALSE;
 	       c = chunk_convert(chunk,&cfmt,DITHER_UNSPEC,bar);
 	       b = wav_save(c,filename,settings,type,dither_mode,bar,fatal);
 	       gtk_object_sink(GTK_OBJECT(c));
@@ -569,6 +583,18 @@ static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
 	  }
      }
 
+     /* If we can't handle it, delegate the FP chunk saving to libsndfile. */
+     if (chunk->format.type == DATAFORMAT_FLOAT &&
+	 !ieee_le_compatible && !ieee_be_compatible) {
+#ifdef HAVE_LIBSNDFILE
+	  return sndfile_save_main(chunk,filename,SF_FORMAT_WAV,dither_mode,
+				   bar,fatal);
+#else
+	  user_error(_("On this system, libsndfile is required to save "
+		     "floating-point wav files."));
+	  return TRUE;
+#endif
+     }
      
      if (fmt->type == DATAFORMAT_PCM && fmt->samplesize==1 && fmt->sign) {
 	  /* user_error(_("8-bit wav-files must be in unsigned format!")); */
@@ -616,12 +642,6 @@ static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
           /* Regular saving */
 
 
-     /* By default, delegate the FP chunk saving to libsndfile. */
-#ifdef HAVE_LIBSNDFILE
-     if (chunk->format.type == DATAFORMAT_FLOAT)
-	  return sndfile_save_main(chunk,filename,SF_FORMAT_WAV,dither_mode,
-				   bar,fatal);
-#endif
 
 	  f = e_fopen(filename,EFILE_WRITE);
 	  if (!f) return TRUE;
@@ -690,7 +710,9 @@ static gboolean sndfile_check(gchar *filename)
      SNDFILE *s;
      s = sf_open(filename, SFM_READ, &info);
      if (s) sf_close(s);
-     return (s!=NULL && ((info.format&SF_FORMAT_SUBMASK) != SF_FORMAT_RAW));
+     return (s!=NULL && ((info.format&SF_FORMAT_SUBMASK) != SF_FORMAT_RAW)
+	     && ((info.format&SF_FORMAT_SUBMASK) != SF_FORMAT_VORBIS ||
+		 inifile_get_guint32("sndfileOggMode",1)!=2));
 }
 
 static Chunk *sndfile_load(gchar *filename, int dither_mode, StatusBar *bar)
@@ -713,20 +735,27 @@ static Chunk *sndfile_load(gchar *filename, int dither_mode, StatusBar *bar)
           return NULL;
      }
      /* printf("info.seekable = %d, info.samples = %d\n",info.seekable,info.samples); */
+     memset(&f,0,sizeof(f));
      f.type = DATAFORMAT_PCM;
-     f.samplerate = info.samplerate;
-     f.channels = info.channels;
      f.bigendian = IS_BIGENDIAN;
      /* Fix samplesize parameter */
      switch (info.format&SF_FORMAT_SUBMASK) {
      case SF_FORMAT_PCM_U8: f.sign=FALSE;f.samplesize=1;raw_readable=TRUE;break;
      case SF_FORMAT_PCM_S8: f.sign=TRUE; f.samplesize=1;raw_readable=TRUE;break;
      case SF_FORMAT_PCM_16: f.sign=TRUE; f.samplesize=2;break;
-	  /*     case SF_FORMAT_PCM_24: f.sign=TRUE; f.samplesize=3;raw_readable=FALSE;break;
-		 case SF_FORMAT_PCM_32: f.sign=TRUE; f.samplesize=4;raw_readable=FALSE;break; */
+     case SF_FORMAT_PCM_24: f.sign=TRUE; f.samplesize=4; f.packing=1; break;
+     case SF_FORMAT_PCM_32: f.sign=TRUE; f.samplesize=4; break;
+
+     case SF_FORMAT_VORBIS:
+	  if (inifile_get_guint32("sndfileOggMode",1)!=0)
+	       info.seekable = 0;
+	  /* (Fall through on purpose) */
+
 	  /* Default to floating point */	  
-     default: f.type = DATAFORMAT_FLOAT; f.samplesize=sizeof(sample_t); break;
+     default: memcpy(&f,&dataformat_sample_t,sizeof(f)); break;
      }
+     f.samplerate = info.samplerate;
+     f.channels = info.channels;
      f.samplebytes = f.channels * f.samplesize;
      ds = gtk_type_new(datasource_get_type());
      ds->type = DATASOURCE_SNDFILE;
@@ -785,6 +814,7 @@ static gboolean find_nearest_sndfile_format(Dataformat *fmt, int format,
      /* Try suggestions first... */
      if (fmt->type == DATAFORMAT_PCM) {
 	  i = fmt->samplesize;
+	  if (fmt->packing != 0) i--;
 	  if (i == 1) i--;
      } else if (fmt->samplesize == sizeof(float)) 
 	  i = 5;
@@ -832,9 +862,12 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
      guint n;
      SNDFILE *s;
      SF_INFO info;
-     sample_t *samplebuf;
+     void *samplebuf;
      ChunkHandle *ch;
      off_t clipcount = 0;
+     Dataformat fmt;
+     Chunk *convchunk;
+     sf_count_t wc;
 
      if (find_nearest_sndfile_format(&(chunk->format),format,&info,filename)) 
 	  return -1;
@@ -847,50 +880,89 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
           return -1;
      }
 
-     sf_command ( s, SFC_SET_NORM_FLOAT, NULL, SF_TRUE );
-     sf_command ( s, SFC_SET_NORM_DOUBLE, NULL, SF_TRUE );
+     /* Decide on a format suitable for passing into one of the
+      * sndfile_write functions (byte,short,int or float) */
+     memcpy(&fmt,&(chunk->format),sizeof(fmt));
+     fmt.bigendian = IS_BIGENDIAN;
+     if (fmt.samplesize > 1) {
+	  fmt.sign = TRUE;
+	  if (fmt.samplesize == 3) {
+	       fmt.samplesize = 4;
+	       fmt.packing = 1;
+	  }
+	  if (fmt.packing == 2) fmt.packing=1;
+	  fmt.samplebytes = fmt.samplesize * fmt.channels;
+     }
+     if (dataformat_samples_equal(&fmt,&(chunk->format))) {
+	  convchunk = chunk;
+	  gtk_object_ref(GTK_OBJECT(convchunk));
+     } else {
+	  convchunk = chunk_convert_sampletype(chunk, &fmt);
+	  gtk_object_ref(GTK_OBJECT(convchunk));
+	  gtk_object_sink(GTK_OBJECT(convchunk));
+     }
 
-     ch = chunk_open(chunk);
+     ch = chunk_open(convchunk);
      if (ch == NULL) {
+	  gtk_object_unref(GTK_OBJECT(convchunk));
 	  sf_close(s);
 	  return -1;
      }
 
-     /* We could speed this up a LOT esp. on output to PCM files. */
-     samplebuf = g_malloc(sizeof(sample_t) * chunk->format.channels * 1024);
+     samplebuf = g_malloc(fmt.samplebytes * 1024);
 
-     for (i=0; i<chunk->length; i+=n) {
-	  /* FIXME: Use chunk_read_array for FP chunks */
-	  n = chunk_read_array_fp(ch,i,1024,samplebuf,dither_mode,&clipcount);
+     for (i=0; i<convchunk->length; i+=n) {
+	  n = chunk_read_array(ch,i,1024*fmt.samplebytes,samplebuf,dither_mode,&clipcount);
 	  if (!n) {
 	       chunk_close(ch);
 	       sf_close(s);
 	       g_free(samplebuf);
+	       gtk_object_unref(GTK_OBJECT(convchunk));
 	       *fatal = TRUE;
 	       return -1;
 	  }
-	  clipcount += unnormalized_count(samplebuf,n*chunk->format.channels);
-	  if (sf_writef_sample_t(s,samplebuf,n) != n) {
+	  n /= fmt.samplebytes;
+	  if (fmt.type == DATAFORMAT_FLOAT) {
+	       if (fmt.samplesize == sizeof(float)) {
+		    wc = sf_writef_float(s,samplebuf,n);
+	       } else {
+		    wc = sf_writef_double(s,samplebuf,n);
+	       }
+	  } else if (fmt.type == DATAFORMAT_PCM) {
+	       if (fmt.samplesize == 1) {
+		    wc = sf_write_raw(s,samplebuf,n*fmt.samplebytes);
+		    wc /= fmt.samplebytes;
+	       } else if (fmt.samplesize == 2) {
+		    wc = sf_writef_short(s,samplebuf,n);
+	       } else {
+		    wc = sf_writef_int(s,samplebuf,n);
+	       }
+	  }
+
+	  if (wc != n) {
 	       c = g_strdup_printf(_("Failed to write to '%s'!"),filename);
 	       user_error(c);
 	       g_free(c);
 	       chunk_close(ch);
 	       sf_close(s);
 	       g_free(samplebuf);
+	       gtk_object_unref(GTK_OBJECT(convchunk));
 	       *fatal = TRUE;
 	       return -1;  
 	  }
-	  if (status_bar_progress(bar, n*chunk->format.samplebytes)) {
+	  if (status_bar_progress(bar, n*fmt.samplebytes)) {
 	       chunk_close(ch);
 	       sf_close(s);
 	       g_free(samplebuf);
 	       xunlink(filename);
+	       gtk_object_unref(GTK_OBJECT(convchunk));
 	       return -1;
 	  }
      }
      chunk_close(ch);
      sf_close(s);
      g_free(samplebuf);
+     gtk_object_unref(GTK_OBJECT(convchunk));
      return clipwarn(clipcount,TRUE);
 
 }
@@ -905,6 +977,16 @@ static gint sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
 
 #endif
 
+gboolean sndfile_ogg_supported(void)
+{
+#if defined(HAVE_LIBSNDFILE)
+     setup_types();
+     return sndfile_ogg_flag;
+#else
+     return FALSE;
+#endif
+}
+
 /* OGG and MP3 */
 
 int run_decoder_cs(gpointer csource, gpointer user_data)
@@ -1009,6 +1091,7 @@ static gboolean ogg_save(Chunk *chunk, gchar *filename, gpointer settings,
 	  memcpy(&fmt,&(chunk->format),sizeof(Dataformat));
 	  fmt.type = DATAFORMAT_PCM;
 	  fmt.samplesize = 4;
+	  fmt.packing = 0;
 	  fmt.samplebytes = fmt.samplesize * fmt.channels;
 	  x = chunk_convert_sampletype(chunk,&fmt);
 	  g_assert(x != NULL);
@@ -1252,7 +1335,7 @@ static Chunk *mp3_load(gchar *filename, int dither_mode, StatusBar *bar)
 static gboolean wav_regular_format(Dataformat *fmt)
 {
      if (fmt->type == DATAFORMAT_FLOAT) return FALSE;
-     return (XOR(fmt->samplesize == 1, fmt->sign));
+     return (XOR(fmt->samplesize == 1, fmt->sign) && fmt->packing == 0);
 }
 
 static gint mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
@@ -1277,9 +1360,10 @@ static gint mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
 				    &clipcount);
      else {
 	  memcpy(&fmt,&(chunk->format),sizeof(Dataformat));
-	  if (fmt.type == DATAFORMAT_FLOAT || fmt.samplesize == 3) {
+	  if (fmt.type == DATAFORMAT_FLOAT || fmt.samplesize == 3 || fmt.packing!=0) {
 	       fmt.type = DATAFORMAT_PCM;
 	       fmt.samplesize = 4;
+	       fmt.packing = 0;
 	  }
 	  fmt.sign = (fmt.samplesize > 1);
 	  fmt.bigendian = FALSE;
