@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2005 2012, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -35,7 +35,7 @@ typedef struct {
 
      GtkTable table;
 
-     Combo *samplesize_combo,*sign_combo,*endian_combo;
+     Combo *samplesize_combo,*sign_combo,*endian_combo,*packing_combo;
      Combo *channel_combo;
      Intbox *rate_box;
 
